import { BadRequestException, Injectable } from '@nestjs/common';
import { FindOptionsWhere } from 'typeorm';
import { PageOptionsRequestDto } from '~/core/dtos';
import { OciService } from '~/document/services';
import { CreateAllowanceChangeRequestDto } from '../dtos/request';
import { AllowanceChangeRequest } from '../entities';
import { AllowanceChangeRequestStatus } from '../enums';
import { AllowanceChangeRequestsRepository } from '../repositories';
import { AllowancesService } from './allowances.service';

@Injectable()
export class AllowanceChangeRequestsService {
  constructor(
    private readonly allowanceChangeRequestsRepository: AllowanceChangeRequestsRepository,
    private readonly ociService: OciService,
    private readonly allowanceService: AllowancesService,
  ) {}

  async createAllowanceChangeRequest(juniorId: string, body: CreateAllowanceChangeRequestDto) {
    const allowance = await this.allowanceService.validateAllowanceForJunior(juniorId, body.allowanceId);

    if (allowance.amount === body.amount) {
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.SAME_AMOUNT');
    }

    const requestWithTheSameAmount = await this.findAllowanceChangeRequestBy({
      allowanceId: body.allowanceId,
      amount: body.amount,
      status: AllowanceChangeRequestStatus.PENDING,
    });

    if (requestWithTheSameAmount) {
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.SAME_AMOUNT_PENDING');
    }

    return this.allowanceChangeRequestsRepository.createAllowanceChangeRequest(body.allowanceId, body);
  }

  findAllowanceChangeRequestBy(where: FindOptionsWhere<AllowanceChangeRequest>) {
    return this.allowanceChangeRequestsRepository.findAllowanceChangeRequestBy(where);
  }

  async approveAllowanceChangeRequest(guardianId: string, requestId: string) {
    const request = await this.findAllowanceChangeRequestBy({ id: requestId, allowance: { guardianId } });

    if (!request) {
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.NOT_FOUND');
    }
    if (request.status === AllowanceChangeRequestStatus.APPROVED) {
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.ALREADY_APPROVED');
    }
    return this.allowanceChangeRequestsRepository.updateAllowanceChangeRequestStatus(
      requestId,
      AllowanceChangeRequestStatus.APPROVED,
    );
  }

  async rejectAllowanceChangeRequest(guardianId: string, requestId: string) {
    const request = await this.findAllowanceChangeRequestBy({ id: requestId, allowance: { guardianId } });

    if (!request) {
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.NOT_FOUND');
    }
    if (request.status === AllowanceChangeRequestStatus.REJECTED) {
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.ALREADY_REJECTED');
    }
    return this.allowanceChangeRequestsRepository.updateAllowanceChangeRequestStatus(
      requestId,
      AllowanceChangeRequestStatus.REJECTED,
    );
  }

  async findAllowanceChangeRequests(
    guardianId: string,
    query: PageOptionsRequestDto,
  ): Promise<[AllowanceChangeRequest[], number]> {
    const [requests, itemCount] = await this.allowanceChangeRequestsRepository.findAllowanceChangeRequests(
      guardianId,
      query,
    );

    await this.prepareAllowanceChangeRequestsImages(requests);

    return [requests, itemCount];
  }

  async findAllowanceChangeRequestById(guardianId: string, requestId: string) {
    const request = await this.allowanceChangeRequestsRepository.findAllowanceChangeRequestBy(
      {
        id: requestId,
        allowance: { guardianId },
      },
      true,
    );

    if (!request) {
      throw new BadRequestException('ALLOWANCE_CHANGE_REQUEST.NOT_FOUND');
    }

    await this.prepareAllowanceChangeRequestsImages([request]);

    return request;
  }

  private prepareAllowanceChangeRequestsImages(requests: AllowanceChangeRequest[]) {
    return Promise.all(
      requests.map(async (request) => {
        const profilePicture = request.allowance.junior.customer.profilePicture;
        if (profilePicture) {
          profilePicture.url = await this.ociService.generatePreSignedUrl(profilePicture);
        }
      }),
    );
  }
}
