import { BadRequestException, Injectable } from '@nestjs/common';
import moment from 'moment';
import { PageOptionsRequestDto } from '~/core/dtos';
import { OciService } from '~/document/services';
import { JuniorService } from '~/junior/services';
import { CreateAllowanceRequestDto } from '../dtos/request';
import { Allowance } from '../entities';
import { AllowancesRepository } from '../repositories';

@Injectable()
export class AllowancesService {
  constructor(
    private readonly allowancesRepository: AllowancesRepository,
    private readonly juniorService: JuniorService,
    private readonly ociService: OciService,
  ) {}

  async createAllowance(guardianId: string, body: CreateAllowanceRequestDto) {
    if (moment(body.startDate).isBefore(moment().startOf('day'))) {
      throw new BadRequestException('ALLOWANCE.START_DATE_BEFORE_TODAY');
    }
    if (moment(body.startDate).isAfter(body.endDate)) {
      throw new BadRequestException('ALLOWANCE.START_DATE_AFTER_END_DATE');
    }

    const doesJuniorBelongToGuardian = await this.juniorService.doesJuniorBelongToGuardian(guardianId, body.juniorId);

    if (!doesJuniorBelongToGuardian) {
      throw new BadRequestException('JUNIOR.DOES_NOT_BELONG_TO_GUARDIAN');
    }

    const allowance = await this.allowancesRepository.createAllowance(guardianId, body);

    return this.findAllowanceById(allowance.id);
  }

  async findAllowanceById(allowanceId: string, guardianId?: string) {
    const allowance = await this.allowancesRepository.findAllowanceById(allowanceId, guardianId);

    if (!allowance) {
      throw new BadRequestException('ALLOWANCE.NOT_FOUND');
    }
    await this.prepareAllowanceDocuments([allowance]);
    return allowance;
  }

  async findAllowances(guardianId: string, query: PageOptionsRequestDto): Promise<[Allowance[], number]> {
    const [allowances, itemCount] = await this.allowancesRepository.findAllowances(guardianId, query);
    await this.prepareAllowanceDocuments(allowances);
    return [allowances, itemCount];
  }

  async deleteAllowance(guardianId: string, allowanceId: string) {
    const { affected } = await this.allowancesRepository.deleteAllowance(guardianId, allowanceId);

    if (!affected) {
      throw new BadRequestException('ALLOWANCE.NOT_FOUND');
    }
  }

  async validateAllowanceForJunior(juniorId: string, allowanceId: string) {
    const allowance = await this.allowancesRepository.findAllowanceById(allowanceId);

    if (!allowance) {
      throw new BadRequestException('ALLOWANCE.NOT_FOUND');
    }

    if (allowance.juniorId !== juniorId) {
      throw new BadRequestException('ALLOWANCE.DOES_NOT_BELONG_TO_JUNIOR');
    }

    return allowance;
  }

  private async prepareAllowanceDocuments(allowance: Allowance[]) {
    await Promise.all(
      allowance.map(async (allowance) => {
        const profilePicture = allowance.junior.customer.profilePicture;
        if (profilePicture) {
          profilePicture.url = await this.ociService.generatePreSignedUrl(profilePicture);
        }
      }),
    );
  }
}
