import { BadRequestException, Injectable } from '@nestjs/common';
import { Roles } from '~/auth/enums';
import { IJwtPayload } from '~/auth/interfaces';
import { OciService } from '~/document/services';
import { JuniorService } from '~/junior/services';
import { CreateGiftRequestDto, GiftFiltersRequestDto, GiftReplyRequestDto } from '../dtos/request';
import { Gift } from '../entities';
import { GiftStatus } from '../enums';
import { GiftsRepository } from '../repositories';

@Injectable()
export class GiftsService {
  constructor(
    private readonly juniorService: JuniorService,
    private readonly giftsRepository: GiftsRepository,
    private readonly ociService: OciService,
  ) {}

  async createGift(guardianId: string, body: CreateGiftRequestDto) {
    const doesJuniorBelongToGuardian = await this.juniorService.doesJuniorBelongToGuardian(
      guardianId,
      body.recipientId,
    );

    if (!doesJuniorBelongToGuardian) {
      throw new BadRequestException('JUNIOR.DOES_NOT_BELONG_TO_GUARDIAN');
    }

    const gift = await this.giftsRepository.create(guardianId, body);

    return this.findUserGiftById({ sub: guardianId, roles: [Roles.GUARDIAN] }, gift.id);
  }

  async findUserGiftById(user: IJwtPayload, giftId: string) {
    const gift = user.roles.includes(Roles.GUARDIAN)
      ? await this.giftsRepository.findGuardianGiftById(user.sub, giftId)
      : await this.giftsRepository.findJuniorGiftById(user.sub, giftId);

    if (!gift) {
      throw new BadRequestException('GIFT.NOT_FOUND');
    }

    await this.prepareGiftImages([gift]);

    return gift;
  }

  async redeemGift(juniorId: string, giftId: string) {
    const gift = await this.giftsRepository.findJuniorGiftById(juniorId, giftId);

    if (!gift) {
      throw new BadRequestException('GIFT.NOT_FOUND');
    }

    if (gift.status === GiftStatus.REDEEMED) {
      throw new BadRequestException('GIFT.ALREADY_REDEEMED');
    }

    return this.giftsRepository.redeemGift(gift);
  }

  async UndoGiftRedemption(juniorId: string, giftId: string) {
    const gift = await this.giftsRepository.findJuniorGiftById(juniorId, giftId);

    if (!gift) {
      throw new BadRequestException('GIFT.NOT_FOUND');
    }

    if (gift.status === GiftStatus.AVAILABLE) {
      throw new BadRequestException('GIFT.NOT_REDEEMED');
    }

    return this.giftsRepository.UndoGiftRedemption(gift);
  }

  async replyToGift(juniorId: string, giftId: string, body: GiftReplyRequestDto) {
    const gift = await this.giftsRepository.findJuniorGiftById(juniorId, giftId);

    if (!gift) {
      throw new BadRequestException('GIFT.NOT_FOUND');
    }

    if (gift.reply) {
      throw new BadRequestException('GIFT.ALREADY_REPLIED');
    }

    return this.giftsRepository.replyToGift(gift, body);
  }

  findGifts(user: IJwtPayload, filters: GiftFiltersRequestDto) {
    return this.giftsRepository.findGifts(user, filters);
  }

  private async prepareGiftImages(gifts: Gift[]) {
    await Promise.all(
      gifts.map(async (gift) => {
        gift.image.url = await this.ociService.generatePreSignedUrl(gift.image);
      }),
    );
  }
}
