import { BadRequestException, forwardRef, Inject, Injectable } from '@nestjs/common';
import { Transactional } from 'typeorm-transactional';
import { Roles } from '~/auth/enums';
import { UserService } from '~/auth/services';
import { PageOptionsRequestDto } from '~/core/dtos';
import { CustomerNotificationSettings } from '~/customer/entities/customer-notification-settings.entity';
import { CustomerService } from '~/customer/services';
import { CreateJuniorRequestDto, SetThemeRequestDto } from '../dtos/request';
import { Junior } from '../entities';
import { JuniorRepository } from '../repositories';
import { JuniorTokenService } from './junior-token.service';

@Injectable()
export class JuniorService {
  constructor(
    private readonly juniorRepository: JuniorRepository,
    private readonly juniorTokenService: JuniorTokenService,
    @Inject(forwardRef(() => UserService)) private readonly userService: UserService,
    @Inject(forwardRef(() => CustomerService)) private readonly customerService: CustomerService,
  ) {}

  @Transactional()
  async createJuniors(body: CreateJuniorRequestDto, guardianId: string) {
    const existingUser = await this.userService.findUser([{ email: body.email }, { phoneNumber: body.phoneNumber }]);

    if (existingUser) {
      throw new BadRequestException('USER.ALREADY_EXISTS');
    }

    const user = await this.userService.createUser({
      email: body.email,
      countryCode: body.countryCode,
      phoneNumber: body.phoneNumber,
      roles: [Roles.JUNIOR],
    });

    await this.customerService.createCustomer(
      {
        firstName: body.firstName,
        lastName: body.lastName,
        dateOfBirth: body.dateOfBirth,
        junior: Junior.create({
          id: user.id,
          guardianId,
          relationship: body.relationship,
          civilIdFrontId: body.civilIdFrontId,
          civilIdBackId: body.civilIdBackId,
        }),
        notificationSettings: new CustomerNotificationSettings(),
      },
      user,
    );

    return this.juniorTokenService.generateToken(user.id);
  }

  async findJuniorById(juniorId: string, withGuardianRelation = false, guardianId?: string) {
    const junior = await this.juniorRepository.findJuniorById(juniorId, withGuardianRelation, guardianId);

    if (!junior) {
      throw new BadRequestException('JUNIOR.NOT_FOUND');
    }
    return junior;
  }

  @Transactional()
  async setTheme(body: SetThemeRequestDto, juniorId: string) {
    const junior = await this.findJuniorById(juniorId);
    if (junior.theme) {
      await this.juniorRepository.removeTheme(junior.theme);
    }
    await this.juniorRepository.setTheme(body, junior);
    return this.juniorRepository.findThemeForJunior(juniorId);
  }

  findJuniorsByGuardianId(guardianId: string, pageOptions: PageOptionsRequestDto) {
    return this.juniorRepository.findJuniorsByGuardianId(guardianId, pageOptions);
  }

  async validateToken(token: string) {
    const juniorId = await this.juniorTokenService.validateToken(token);

    return this.findJuniorById(juniorId, true);
  }

  generateToken(juniorId: string) {
    return this.juniorTokenService.generateToken(juniorId);
  }

  async doesJuniorBelongToGuardian(guardianId: string, juniorId: string) {
    const junior = await this.findJuniorById(juniorId, false, guardianId);

    return !!junior;
  }
}
