import { BadRequestException, Injectable } from '@nestjs/common';
import { OciService } from '~/document/services';
import { JuniorService } from '~/junior/services';
import { CreateMoneyRequestRequestDto, MoneyRequestsFiltersRequestDto } from '../dtos/request';
import { MoneyRequest } from '../entities';
import { MoneyRequestFrequency, MoneyRequestStatus } from '../enums';
import { MoneyRequestsRepository } from '../repositories';

@Injectable()
export class MoneyRequestsService {
  constructor(
    private readonly moneyRequestsRepository: MoneyRequestsRepository,
    private readonly juniorService: JuniorService,
    private readonly ociService: OciService,
  ) {}

  async createMoneyRequest(userId: string, body: CreateMoneyRequestRequestDto) {
    if (body.frequency === MoneyRequestFrequency.ONE_TIME) {
      delete body.endDate;
    }

    if (body.endDate && new Date(body.endDate) < new Date()) {
      throw new BadRequestException('MONEY_REQUEST.END_DATE_IN_THE_PAST');
    }

    const junior = await this.juniorService.findJuniorById(userId, true);

    const moneyRequest = await this.moneyRequestsRepository.createMoneyRequest(junior.id, junior.guardianId, body);

    return this.findMoneyRequestById(moneyRequest.id);
  }

  async findMoneyRequestById(moneyRequestId: string, reviewerId?: string) {
    const moneyRequest = await this.moneyRequestsRepository.findMoneyRequestById(moneyRequestId, reviewerId);

    if (!moneyRequest) {
      throw new BadRequestException('MONEY_REQUEST.NOT_FOUND');
    }

    await this.prepareMoneyRequestDocument([moneyRequest]);

    return moneyRequest;
  }

  async findMoneyRequests(
    ReviewerId: string,
    filters: MoneyRequestsFiltersRequestDto,
  ): Promise<[MoneyRequest[], number]> {
    const [moneyRequests, itemCount] = await this.moneyRequestsRepository.findMoneyRequests(ReviewerId, filters);
    await this.prepareMoneyRequestDocument(moneyRequests);
    return [moneyRequests, itemCount];
  }

  async approveMoneyRequest(moneyRequestId: string, reviewerId: string) {
    await this.validateMoneyRequestForReview(moneyRequestId, reviewerId);
    await this.moneyRequestsRepository.updateMoneyRequestStatus(
      moneyRequestId,
      reviewerId,
      MoneyRequestStatus.APPROVED,
    );

    //@TODO send notification and update junior balance
  }

  async rejectMoneyRequest(moneyRequestId: string, reviewerId: string) {
    await this.validateMoneyRequestForReview(moneyRequestId, reviewerId);

    await this.moneyRequestsRepository.updateMoneyRequestStatus(
      moneyRequestId,
      reviewerId,
      MoneyRequestStatus.REJECTED,
    );

    //@TODO send notification
  }

  private async prepareMoneyRequestDocument(moneyRequests: MoneyRequest[]) {
    await Promise.all(
      moneyRequests.map(async (moneyRequest) => {
        const profilePicture = moneyRequest.requester.customer.profilePicture;

        if (profilePicture) {
          profilePicture.url = await this.ociService.generatePreSignedUrl(profilePicture);
        }
      }),
    );
  }

  private async validateMoneyRequestForReview(moneyRequestId: string, reviewerId: string) {
    const moneyRequest = await this.moneyRequestsRepository.findMoneyRequestById(moneyRequestId, reviewerId);

    if (!moneyRequest) {
      throw new BadRequestException('MONEY_REQUEST.NOT_FOUND');
    }
    if (moneyRequest.status !== MoneyRequestStatus.PENDING) {
      throw new BadRequestException('MONEY_REQUEST.ALREADY_REVIEWED');
    }
  }
}
