import { BadRequestException, Injectable } from '@nestjs/common';
import moment from 'moment';
import { PageOptionsRequestDto } from '~/core/dtos';
import { OciService } from '~/document/services';
import { CreateGoalRequestDto, FundGoalRequestDto } from '../dtos/request';
import { SavingGoal } from '../entities';
import { IGoalStats } from '../interfaces';
import { SavingGoalsRepository } from '../repositories';
import { CategoryService } from './category.service';
@Injectable()
export class SavingGoalsService {
  constructor(
    private readonly savingGoalsRepository: SavingGoalsRepository,
    private readonly categoryService: CategoryService,
    private readonly ociService: OciService,
  ) {}

  async createGoal(juniorId: string, body: CreateGoalRequestDto) {
    if (moment(body.dueDate).isBefore(moment())) {
      throw new BadRequestException('GOAL.DUE_DATE_MUST_BE_IN_THE_FUTURE');
    }

    const categories = await this.categoryService.findCategoryByIds(body.categoryIds, juniorId);

    const createdGoal = await this.savingGoalsRepository.createGoal(juniorId, body, categories);

    return this.findGoalById(juniorId, createdGoal.id);
  }

  async findGoals(juniorId: string, pageOptions: PageOptionsRequestDto): Promise<[SavingGoal[], number]> {
    const [goals, itemCount] = await this.savingGoalsRepository.findGoals(juniorId, pageOptions);
    await this.prepareGoalsImages(goals);

    return [goals, itemCount];
  }

  async findGoalById(juniorId: string, goalId: string) {
    const goal = await this.savingGoalsRepository.findGoalById(juniorId, goalId);

    if (!goal) {
      throw new BadRequestException('GOAL.NOT_FOUND');
    }

    await this.prepareGoalsImages([goal]);

    return goal;
  }

  async fundGoal(juniorId: string, goalId: string, body: FundGoalRequestDto) {
    const goal = await this.savingGoalsRepository.findGoalById(juniorId, goalId);

    if (!goal) {
      throw new BadRequestException('GOAL.NOT_FOUND');
    }

    if (goal.currentAmount + body.fundAmount > goal.targetAmount) {
      throw new BadRequestException('GOAL.FUND_EXCEEDS_TOTAL_AMOUNT');
    }

    await this.savingGoalsRepository.fundGoal(juniorId, goalId, body.fundAmount);
  }

  async getStats(juniorId: string): Promise<IGoalStats> {
    const result = await this.savingGoalsRepository.getStats(juniorId);
    return {
      totalTarget: result?.totalTarget,
      totalSaved: result?.totalSaved,
    };
  }

  private async prepareGoalsImages(goals: SavingGoal[]) {
    await Promise.all(
      goals.map(async (goal) => {
        if (goal.imageId) {
          goal.image.url = await this.ociService.generatePreSignedUrl(goal.image);
        }
      }),
    );
  }
}
