import { BadRequestException, Injectable } from '@nestjs/common';
import moment from 'moment';
import { FindOptionsWhere } from 'typeorm';
import { IJwtPayload } from '~/auth/interfaces';
import { OciService } from '~/document/services';
import { CreateTaskRequestDto, TasksFilterOptions, TaskSubmissionRequestDto } from '../dtos/request';
import { Task } from '../entities';
import { SubmissionStatus, TaskStatus } from '../enums';
import { TaskRepository } from '../repositories';

@Injectable()
export class TaskService {
  constructor(private readonly taskRepository: TaskRepository, private readonly ociService: OciService) {}
  async createTask(userId: string, body: CreateTaskRequestDto) {
    if (moment(body.dueDate).isBefore(moment(body.startDate))) {
      throw new BadRequestException('TASK.DUE_DATE_BEFORE_START_DATE');
    }

    if (moment(body.dueDate).isBefore(moment())) {
      throw new BadRequestException('TASK.DUE_DATE_IN_PAST');
    }
    const task = await this.taskRepository.createTask(userId, body);
    return this.findTask({ id: task.id });
  }

  async findTask(where: FindOptionsWhere<Task>) {
    const task = await this.taskRepository.findTask(where);

    if (!task) {
      throw new BadRequestException('TASK.NOT_FOUND');
    }

    await this.prepareTasksPictures([task]);

    return task;
  }

  async findTasks(user: IJwtPayload, query: TasksFilterOptions): Promise<[Task[], number]> {
    const [tasks, count] = await this.taskRepository.findTasks(user, query);
    await this.prepareTasksPictures(tasks);

    return [tasks, count];
  }
  async submitTask(userId: string, taskId: string, body: TaskSubmissionRequestDto) {
    const task = await this.findTask({ id: taskId, assignedToId: userId });

    if (task.status == TaskStatus.COMPLETED) {
      throw new BadRequestException('TASK.ALREADY_COMPLETED');
    }

    if (task.isProofRequired && !body.imageId) {
      throw new BadRequestException('TASK.PROOF_REQUIRED');
    }

    await this.taskRepository.createSubmission(task, body);
  }

  async approveTaskSubmission(userId: string, taskId: string) {
    const task = await this.findTask({ id: taskId, assignedById: userId });

    if (!task.submission) {
      throw new BadRequestException('TASK.NO_SUBMISSION');
    }

    if (task.submission.status == SubmissionStatus.APPROVED) {
      throw new BadRequestException('TASK.SUBMISSION_ALREADY_REVIEWED');
    }

    await this.taskRepository.approveSubmission(task.submission);
  }

  async rejectTaskSubmission(userId: string, taskId: string) {
    const task = await this.findTask({ id: taskId, assignedById: userId });

    if (!task.submission) {
      throw new BadRequestException('TASK.NO_SUBMISSION');
    }

    if (task.submission.status == SubmissionStatus.REJECTED) {
      throw new BadRequestException('TASK.SUBMISSION_ALREADY_REVIEWED');
    }

    await this.taskRepository.rejectSubmission(task.submission);
  }

  async prepareTasksPictures(tasks: Task[]) {
    await Promise.all(
      tasks.map(async (task) => {
        const [imageUrl, submissionUrl, profilePictureUrl] = await Promise.all([
          this.ociService.generatePreSignedUrl(task.image),
          this.ociService.generatePreSignedUrl(task.submission?.proofOfCompletion),
          this.ociService.generatePreSignedUrl(task.assignedTo.customer.profilePicture),
        ]);

        task.image.url = imageUrl;

        if (task.submission?.proofOfCompletion) {
          task.submission.proofOfCompletion.url = submissionUrl;
        }

        if (task.assignedTo.customer.profilePicture) {
          task.assignedTo.customer.profilePicture.url = profilePictureUrl;
        }
      }),
    );
  }
}
